@@ -8,11 +8,11 @@ import time
 from asyncio import CancelledError
 from collections.abc import AsyncGenerator, Coroutine, Generator
 from typing import Any, NoReturn, cast
+from unittest import mock
 
 import pytest
 from exceptiongroup import catch
-from pytest import FixtureRequest
-from pytest_mock import MockerFixture
+from pytest import FixtureRequest, MonkeyPatch
 
 import anyio
 from anyio import (
@@ -32,6 +32,8 @@ from anyio import (
 from anyio.abc import TaskGroup, TaskStatus
 from anyio.lowlevel import checkpoint
 
+from .conftest import asyncio_params, no_other_refs
+
 if sys.version_info < (3, 11):
     from exceptiongroup import BaseExceptionGroup, ExceptionGroup
 
@@ -200,7 +202,7 @@ async def test_start_cancelled() -> None:
     assert not finished
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_start_native_host_cancelled() -> None:
     started = finished = False
 
@@ -224,7 +226,7 @@ async def test_start_native_host_cancelled() -> None:
     assert not finished
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_start_native_child_cancelled() -> None:
     task = None
     finished = False
@@ -248,7 +250,7 @@ async def test_start_native_child_cancelled() -> None:
     assert not finished
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_propagate_native_cancellation_from_taskgroup() -> None:
     async def taskfunc() -> None:
         async with create_task_group() as tg:
@@ -261,32 +263,44 @@ async def test_propagate_native_cancellation_from_taskgroup() -> None:
         await task
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
-async def test_cancel_with_nested_task_groups(mocker: MockerFixture) -> None:
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
+async def test_cancel_with_nested_task_groups() -> None:
     """Regression test for #695."""
 
     async def shield_task() -> None:
         with CancelScope(shield=True) as scope:
-            shielded_cancel_spy = mocker.spy(scope, "_deliver_cancellation")
-            await sleep(0.5)
+            with mock.patch.object(
+                scope,
+                "_deliver_cancellation",
+                wraps=getattr(scope, "_deliver_cancellation"),
+            ) as shielded_cancel_spy:
+                await sleep(0.5)
 
-            assert len(outer_cancel_spy.call_args_list) < 10
-            shielded_cancel_spy.assert_not_called()
+                assert len(outer_cancel_spy.call_args_list) < 10
+                shielded_cancel_spy.assert_not_called()
 
     async def middle_task() -> None:
         try:
             async with create_task_group() as tg:
-                middle_cancel_spy = mocker.spy(tg.cancel_scope, "_deliver_cancellation")
-                tg.start_soon(shield_task, name="shield task")
+                with mock.patch.object(
+                    tg.cancel_scope,
+                    "_deliver_cancellation",
+                    wraps=getattr(tg.cancel_scope, "_deliver_cancellation"),
+                ) as middle_cancel_spy:
+                    tg.start_soon(shield_task, name="shield task")
         finally:
             assert len(middle_cancel_spy.call_args_list) < 10
             assert len(outer_cancel_spy.call_args_list) < 10
 
     async with create_task_group() as tg:
-        outer_cancel_spy = mocker.spy(tg.cancel_scope, "_deliver_cancellation")
-        tg.start_soon(middle_task, name="middle task")
-        await wait_all_tasks_blocked()
-        tg.cancel_scope.cancel()
+        with mock.patch.object(
+            tg.cancel_scope,
+            "_deliver_cancellation",
+            wraps=getattr(tg.cancel_scope, "_deliver_cancellation"),
+        ) as outer_cancel_spy:
+            tg.start_soon(middle_task, name="middle task")
+            await wait_all_tasks_blocked()
+            tg.cancel_scope.cancel()
 
     assert len(outer_cancel_spy.call_args_list) < 10
 
@@ -691,7 +705,7 @@ async def test_shielded_cleanup_after_cancel() -> None:
             assert get_current_task().has_pending_cancellation()
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_cleanup_after_native_cancel() -> None:
     """Regression test for #832."""
     # See also https://github.com/python/cpython/pull/102815.
@@ -791,7 +805,7 @@ async def test_empty_taskgroup_contains_yield_point() -> None:
         assert outer_task_ran
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_cancel_host_asyncgen() -> None:
     done = False
 
@@ -853,7 +867,8 @@ async def test_shielding_mutate() -> None:
             completed = True
             scope.shield = False
             await sleep(1)
-            pytest.fail("Execution should not reach this point")
+
+        pytest.fail("Execution should not reach this point")
 
     async with create_task_group() as tg:
         await tg.start(task)
@@ -1146,7 +1161,7 @@ def test_cancel_generator_based_task() -> None:
 @pytest.mark.filterwarnings(
     'ignore:"@coroutine" decorator is deprecated:DeprecationWarning'
 )
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_schedule_old_style_coroutine_func() -> None:
     """
     Test that we give a sensible error when a user tries to spawn a task from a
@@ -1169,7 +1184,7 @@ async def test_schedule_old_style_coroutine_func() -> None:
             tg.start_soon(corofunc)
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_cancel_native_future_tasks() -> None:
     async def wait_native_future() -> None:
         loop = asyncio.get_running_loop()
@@ -1180,7 +1195,7 @@ async def test_cancel_native_future_tasks() -> None:
         tg.cancel_scope.cancel()
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_cancel_native_future_tasks_cancel_scope() -> None:
     async def wait_native_future() -> None:
         with anyio.CancelScope():
@@ -1192,7 +1207,7 @@ async def test_cancel_native_future_tasks_cancel_scope() -> None:
         tg.cancel_scope.cancel()
 
 
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 async def test_cancel_completed_task() -> None:
     loop = asyncio.get_running_loop()
     old_exception_handler = loop.get_exception_handler()
@@ -1288,7 +1303,7 @@ async def test_cancelscope_exit_before_enter() -> None:
 
 
 @pytest.mark.parametrize(
-    "anyio_backend", ["asyncio"]
+    "anyio_backend", asyncio_params
 )  # trio does not check for this yet
 async def test_cancelscope_exit_in_wrong_task() -> None:
     async def enter_scope(scope: CancelScope) -> None:
@@ -1403,7 +1418,7 @@ async def test_start_parent_id() -> None:
     sys.version_info < (3, 11),
     reason="Task uncancelling is only supported on Python 3.11",
 )
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
 class TestUncancel:
     async def test_uncancel_after_native_cancel(self) -> None:
         task = cast(asyncio.Task, asyncio.current_task())
@@ -1602,16 +1617,6 @@ async def test_start_cancels_parent_scope() -> None:
     assert not tg.cancel_scope.cancel_called
 
 
-if sys.version_info <= (3, 11):
-
-    def no_other_refs() -> list[object]:
-        return [sys._getframe(1)]
-else:
-
-    def no_other_refs() -> list[object]:
-        return []
-
-
 @pytest.mark.skipif(
     sys.implementation.name == "pypy",
     reason=(
@@ -1764,17 +1769,56 @@ class TestTaskStatusTyping:
     reason="Eager task factories require Python 3.12",
 )
 @pytest.mark.parametrize("anyio_backend", ["asyncio"])
-async def test_eager_task_factory(request: FixtureRequest) -> None:
+@pytest.mark.parametrize("use_custom_eager_factory", [False, True])
+async def test_eager_task_factory(
+    request: FixtureRequest, use_custom_eager_factory: bool
+) -> None:
+    ran = False
+
     async def sync_coro() -> None:
+        nonlocal ran
+        ran = True
+
         # This should trigger fetching the task state
         with CancelScope():  # noqa: ASYNC100
             pass
 
+    def create_custom_task(
+        coro: Coroutine[Any, Any, Any], /, **kwargs: Any
+    ) -> asyncio.Task[Any]:
+        return asyncio.Task(coro, **kwargs)
+
     loop = asyncio.get_running_loop()
     old_task_factory = loop.get_task_factory()
-    loop.set_task_factory(asyncio.eager_task_factory)
+    if use_custom_eager_factory:
+        loop.set_task_factory(asyncio.create_eager_task_factory(create_custom_task))
+    else:
+        loop.set_task_factory(asyncio.eager_task_factory)
+
     request.addfinalizer(lambda: loop.set_task_factory(old_task_factory))
 
     async with create_task_group() as tg:
         tg.start_soon(sync_coro)
+        assert not ran
         tg.cancel_scope.cancel()
+
+
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
+async def test_patched_asyncio_task(monkeypatch: MonkeyPatch) -> None:
+    monkeypatch.setattr(
+        asyncio,
+        "Task",
+        asyncio.tasks._PyTask,  # type: ignore[attr-defined]
+    )
+    async with create_task_group() as tg:
+        tg.start_soon(sleep, 0)
+
+
+async def test_exception_groups_suppresses_exc_context() -> None:
+    with pytest.raises(
+        cast(type[ExceptionGroup[Exception]], ExceptionGroup)
+    ) as exc_info:
+        async with create_task_group():
+            raise Exception("Error")
+
+    assert exc_info.value.__suppress_context__
